// SPDX-FileCopyrightText: Fondazione Istituto Italiano di Tecnologia (IIT)
// SPDX-License-Identifier: BSD-3-Clause

#ifndef IDYNTREE_ESTIMATION_SIMPLE_LEGGED_ODOMETRY2_
#define IDYNTREE_ESTIMATION_SIMPLE_LEGGED_ODOMETRY2_

#ifdef __DEPRECATED
  #warning <iDynTree/Estimation/SimpleLeggedOdometry.h> is deprecated. Please use <iDynTree/SimpleLeggedOdometry.h>. To disable this warning use -Wno-deprecated.
#endif

#include <iDynTree/SimpleLeggedOdometry.h>

#endif
